#===============================================================================
#
# Copyright (c) 2014 Qualcomm Technologies, Inc. All Rights Reserved.
# Qualcomm Technologies Proprietary and Confidential.
#
#===============================================================================

import struct
import defines

from sectools.features.fuse_common.parsegen.secdat.secdatbase import SecdatParseGen
from sectools.common.utils.c_logging import logger


class SecdatParseGenRevOne(SecdatParseGen):

    def __init__(self, header_info='', header_version=None,
                 fuse_list_header_version=None):
        SecdatParseGen.__init__(self, header_info, header_version,
                                fuse_list_header_version)

    #--------------------------------------------------------------------------
    # Mandatory Properties overloaded from the base class
    #--------------------------------------------------------------------------
    @classmethod
    def is_plugin(cls):
        return True

    @classmethod
    def file_version(cls):
        return defines.SECDAT_REVISION_ONE

    def secdat_input_parser(self, input_secdat=None, feature=None):
        return None, None

    def create_header(self, data_size, info, feature):
        header = _fuseprov_secdat_hdr_type()
        header.magic1 = defines.FUSEPROV_SECDAT_MAGIC1
        header.magic2 = defines.FUSEPROV_SECDAT_MAGIC2
        header.revision = defines.SECDAT_REVISION_ONE
        header.size = data_size
        header.info = info.ljust(defines.FUSEPROV_INFO_MAX_SIZE, '\0')
        return header

    def validate_header(self, header, file_size):
        if not (header.magic1 == defines.FUSEPROV_SECDAT_MAGIC1 and
                header.magic2 == defines.FUSEPROV_SECDAT_MAGIC2 and
                header.revision == defines.SECDAT_REVISION_ONE and
                (header.size + header.PACK_SIZE == file_size)):
            raise RuntimeError('Header Mismatch')
        pass

    def parse_secdat_hdr_data(self, file_data):

        file_data_len = len(file_data)

        header_data = file_data[:_fuseprov_secdat_hdr_type.PACK_SIZE]
        file_data = file_data[_fuseprov_secdat_hdr_type.PACK_SIZE:]
        self._header = _fuseprov_secdat_hdr_type(header_data)
        self.validate_header(self._header, file_data_len)
        self.header_info = self._header.info
        self.header_version = self._header.revision
        return header_data, file_data

    def write(self, data_model, input_secdat, secdat, common_secdat, exist_secdat, feature):

        # Check whether sec.dat is passed as input to fuseblower sec.dat revision-1
        if input_secdat is not None:
            if exist_secdat is not None:
                self.update_from_data_model(data_model, feature)
                self.generate(secdat)
            else:
                raise RuntimeError('Passing sec.dat as input to Fuseblower_Sec.dat_Revision-1' '\n'
                                   'Please remove sec.dat as input and try again')

        # To generate the fuseblower sec.dat revision-1
        else:
            self.update_from_data_model(data_model, feature)
            self.generate(secdat)

    def secdat_repr(self, file_path, seg = None, feature = None):

        retval = ''
        retval += ('Secdat properties: ' + '\n' + '\n'
                   '' + 'File path: ' + file_path + '\n' + '\n'
                   '' + 'Header: ' + '\n'
                   '' + logger.increase_indent(repr(self._header)) + '\n'
                   '' + 'Fuse Header: ' + '\n'
                   '' + logger.increase_indent(repr(self._fuse_header)) + '\n')

        for fuse in self._fuse_list:
            retval += ('' + 'Fuse: ' + '\n'
                       '' + logger.increase_indent(repr(fuse)) + '\n')


        retval += ('Footer: ' + '\n'
                   '' + logger.increase_indent(repr(self._footer)) + '\n')
        return retval.strip()


class _fuseprov_secdat_hdr_type(object):

    # Structure object to align and package the header
    PACK_FMT = struct.Struct('IIII' +
                             'B' * defines.FUSEPROV_INFO_MAX_SIZE +
                             'I' * defines.FUSEPROV_RESERVED_MAX_SIZE)

    # Size of the header
    PACK_SIZE = ((4 * 4) +
                (1 * defines.FUSEPROV_INFO_MAX_SIZE) +
                (4 * defines.FUSEPROV_RESERVED_MAX_SIZE))

    def __init__(self, data=None):
        if data is None:
            data = ('{0:c}'.format(0))*self.PACK_SIZE
        unpacked_data = (self.PACK_FMT).unpack(data)
        self.magic1 = unpacked_data[0]
        self.magic2 = unpacked_data[1]
        self.revision = unpacked_data[2]
        self.size = unpacked_data[3]
        self.info = ''
        self.reserved = ''
        # Pull info
        i = 4
        for i in range(i, i + defines.FUSEPROV_INFO_MAX_SIZE):
            self.info += '{0:c}'.format(unpacked_data[i])
        # Pull reserved
        i += 1
        for i in range(i, i + defines.FUSEPROV_RESERVED_MAX_SIZE):
            self.reserved += '{0:c}'.format(unpacked_data[i])
        pass

    def __repr__(self):
        retval = ''
        for attr in sorted(self.__dict__.keys()):
            value = self.__dict__[attr]
            retval += (str(attr) + ': ' + (value if isinstance(value, str) else
                       str(hex(value))) + '\n')
        return (self.__class__.__name__ + ' Properties: ' + '\n' +
                logger.increase_indent(retval))

    def get_packed_data(self):
        values = ([self.magic1,
                   self.magic2,
                   self.revision,
                   self.size] +
                  [ord(c) for c in self.info] +
                  [ord(c) for c in self.reserved]
                 )
        return (self.PACK_FMT).pack(*values)
